@@ -1,17 +1,17 @@
 #include <gmp.h>
 #include "macros.h"
 #include "utils.h"
-#include "permutations-ordinary.h"
-#include "permutations-k.h"
-#include "permutations-multiset.h"
-#include "permutations-replacement.h"
+#include "permutations/permutations-ordinary.c"
+#include "permutations/permutations-k.c"
+#include "permutations/permutations-multiset.c"
+#include "permutations/permutations-replacement.c"
 
 
 SEXP npermutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _replace, SEXP _bigz) {
     int i;
     SEXP ans;
 
-    int has_labels = !Rf_isNull(_v);
+    int has_vector = !Rf_isNull(_v);
     int multiset = !Rf_isNull(_freq);
 
     int n, k;
@@ -21,6 +21,10 @@ SEXP npermutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _replace
 
     VALIDATE_ARGUMENTS();
 
+    if (multiset && replace) {
+        n = flen;
+    }
+
     if (Rf_asLogical(_bigz)) {
         mpz_t z;
         mpz_init(z);
@@ -72,12 +76,12 @@ SEXP npermutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _replace
 }
 
 
-SEXP collect_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _replace,
+SEXP get_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _replace,
                       SEXP _layout, SEXP _d, SEXP _index, SEXP _nsample, SEXP state, SEXP _skip, SEXP _drop) {
     int i;
     SEXP ans = R_NilValue;
 
-    int has_labels = !Rf_isNull(_v);
+    int has_vector = !Rf_isNull(_v);
     int multiset = !Rf_isNull(_freq);
 
     int n, k;
@@ -93,21 +97,13 @@ SEXP collect_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _
     if (Rf_isNull(_index) && Rf_isNull(_nsample)) {
         if (k == 0) {
             if (layout == 'r') {
-                if (has_labels) {
-                    ans = Rf_allocMatrix(TYPEOF(_v), 1, 0);
-                } else {
-                    ans = Rf_allocMatrix(INTSXP, 1, 0);
-                }
+                ans = Rf_allocMatrix(has_vector ? TYPEOF(_v) : INTSXP, 1, 0);
             } else if (layout == 'c') {
-                if (has_labels) {
-                    ans = Rf_allocMatrix(TYPEOF(_v), 0, 1);
-                } else {
-                    ans = Rf_allocMatrix(INTSXP, 0, 1);
-                }
+                ans = Rf_allocMatrix(has_vector ? TYPEOF(_v) : INTSXP, 0, 1);
             } else if (layout == 'l') {
                 if (n == 0) {
                     ans = PROTECT(Rf_allocVector(VECSXP, 1));
-                    SEXP ansi = PROTECT(Rf_allocVector(has_labels ? TYPEOF(_v) : INTSXP, 0));
+                    SEXP ansi = PROTECT(Rf_allocVector(has_vector ? TYPEOF(_v) : INTSXP, 0));
                     SET_VECTOR_ELT(ans, 0, ansi);
                     UNPROTECT(2);
                 } else {
@@ -116,17 +112,9 @@ SEXP collect_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _
             }
         } else if (k > n && (!replace || n == 0)) {
             if (layout == 'r') {
-                if (has_labels) {
-                    ans = Rf_allocMatrix(TYPEOF(_v), 0, k);
-                } else {
-                    ans = Rf_allocMatrix(INTSXP, 0, k);
-                }
+                ans = Rf_allocMatrix(has_vector ? TYPEOF(_v) : INTSXP, 0, k);
             } else if (layout == 'c') {
-                if (has_labels) {
-                    ans = Rf_allocMatrix(TYPEOF(_v), k, 0);
-                } else {
-                    ans = Rf_allocMatrix(INTSXP, k, 0);
-                }
+                ans = Rf_allocMatrix(has_vector ? TYPEOF(_v) : INTSXP, k, 0);
             } else if (layout == 'l') {
                 ans = Rf_allocVector(VECSXP, 0);
             }
@@ -141,23 +129,34 @@ SEXP collect_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _
         }
     } else {
         if (replace) {
-            ans = catch_replacement_permutations(n, k, _v, layout, _index, _nsample);
+            ans = draw_replacement_permutations(n, k, _v, layout, _index, _nsample);
         } else if (multiset) {
-            ans = catch_multiset_permutations(fp, flen, k, _v, layout, _index, _nsample);
+            ans = draw_multiset_permutations(fp, flen, k, _v, layout, _index, _nsample);
         } else if (n == k) {
-            ans = catch_ordinary_permutations(n, _v, layout, _index, _nsample);
+            ans = draw_ordinary_permutations(n, _v, layout, _index, _nsample);
         } else {
-            ans = catch_k_permutations(n, k, _v, layout, _index, _nsample);
+            ans = draw_k_permutations(n, k, _v, layout, _index, _nsample);
         }
     }
 
     PROTECT(ans);
     attach_factor_levels(ans, _v);
-    if ((!Rf_isNull(_drop) && Rf_asLogical(_drop)) || ((Rf_isNull(_drop) || Rf_asLogical(_drop)) &&
-                ((d == 1 && Rf_isNull(_layout)) ||
-                    (!Rf_isNull(_index) && index_length(_index) == 1 && Rf_isNull(_layout)) ||
-                    (!Rf_isNull(_nsample) && as_uint(_nsample) == 1 && Rf_isNull(_layout)))
-                )) {
+    if (d > 0 && !Rf_isNull(state)) {
+        if ((layout == 'r' && (Rf_nrows(ans) == 0)) ||
+                (layout == 'c' && Rf_ncols(ans) == 0) ||
+                (layout == 'l' && Rf_length(ans) == 0)) {
+            ans = R_NilValue;
+        } else if (k == 0 ||
+                (layout == 'r' && (Rf_nrows(ans) < d)) ||
+                (layout == 'c' && Rf_ncols(ans) < d) ||
+                (layout == 'l' && Rf_length(ans) < d)) {
+            Rf_defineVar(Rf_install("null_pending"), Rf_ScalarLogical(1), state);
+        }
+    }
+    if ((!Rf_isNull(_drop) && Rf_asLogical(_drop)) ||
+            ((Rf_isNull(_drop) || Rf_asLogical(_drop)) && ((d == 1 && Rf_isNull(_layout)) ||
+            (!Rf_isNull(_index) && index_length(_index) == 1 && Rf_isNull(_layout)) ||
+            (!Rf_isNull(_nsample) && as_uint(_nsample) == 1 && Rf_isNull(_layout))) )) {
         if (layout == 'r' && Rf_nrows(ans) == 1) {
             Rf_setAttrib(ans, R_DimSymbol, R_NilValue);
         } else if (layout == 'c' && Rf_ncols(ans) == 1) {
@@ -166,18 +165,6 @@ SEXP collect_permutations(SEXP _x, SEXP _k, SEXP _n, SEXP _v, SEXP _freq, SEXP _
             ans = VECTOR_ELT(ans, 0);
         }
     }
-
-    if (d > 0 && !Rf_isNull(state)) {
-        if ((layout == 'r' && (Rf_nrows(ans) == 0)) ||
-                        (layout == 'c' && Rf_ncols(ans) == 0) ||
-                        (layout == 'l' && Rf_length(ans) == 0)) {
-                    ans = R_NilValue;
-        } else if ((layout == 'r' && (Rf_nrows(ans) < d)) ||
-                (layout == 'c' && Rf_ncols(ans) < d) ||
-                (layout == 'l' && Rf_length(ans) < d)) {
-            Rf_defineVar(Rf_install("null_pending"), Rf_ScalarLogical(1), state);
-        }
-    }
     UNPROTECT(1);
     return ans;
 }
